<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use App\Repositories\AccountRepository;
use App\Http\Requests\SaveAccountRequest;
use Throwable;

class AccountController extends Controller
{
    protected $accountRepository;

    public function __construct(AccountRepository $accountRepository)
    {
        $this->accountRepository = $accountRepository;
    }

    public function all()
    {
        try {
            $accounts = $this->accountRepository->all();

            return response()->json($accounts, 200);
            
        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function create(SaveAccountRequest $request)
    {
        try {
            DB::beginTransaction();

            $request['by_user'] = Auth::user()->id;

            $account = $this->accountRepository->save($request);

            DB::commit();

            return response()->json([
                'message' => 'Account created successfully.',
                'data' => $account
            ], 201);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function edit(SaveAccountRequest $request)
    {
        try {
            DB::beginTransaction();

            $account = $this->accountRepository->getById($request->id);

            if (empty($account)) {
                return response()->json([
                    'message' => 'Account not found.'
                ], 404);
            }

            $request['by_user'] = Auth::user()->id;

            $accountUpdated = $this->accountRepository->update($account, $request);

            DB::commit();

            return response()->json([
                'message' => 'Account successfully updated.',
                'data' => $accountUpdated
            ], 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function delete(Request $request)
    {
        try {
            DB::beginTransaction();

            $account = $this->accountRepository->getById($request->byId);

            if (empty($account)) {
                return response()->json([
                    'message' => 'Account not found.'
                ], 404);
            }

            $accountDeleted = $this->accountRepository->delete($account, Auth::user()->id);

            DB::commit();

            return response()->json([
                'message' => 'Account successfully deleted.',
                'data' => $accountDeleted
            ], 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }
}
