<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use App\Repositories\NodeRepository;
use App\Http\Requests\SaveNodeRequest;
use Throwable;

class NodeController extends Controller
{
    protected $nodeRepository;

    public function __construct(NodeRepository $nodeRepository)
    {
        $this->nodeRepository = $nodeRepository;
    }

    public function all()
    {
        try {
            $nodesData = $this->nodeRepository->all();
            $nodes = $this->getAllNodes($nodesData);

            return response()->json($nodes, 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function create(SaveNodeRequest $request)
    {
        try {
            DB::beginTransaction();

            $request['by_user'] = Auth::user()->id;

            $node = $this->nodeRepository->save($request);

            DB::commit();

            return response()->json([
                'message' => 'Node successfully created.',
                'data' => $node,
            ], 201);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function edit(SaveNodeRequest $request)
    {
        try {
            DB::beginTransaction();

            $request['by_user'] = Auth::user()->id;

            $node = $this->nodeRepository->update($request);

            DB::commit();

            return response()->json([
                'message' => 'Node successfully updated.',
                'date' => $node,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function delete(Request $request)
    {
        try {
            $node = $this->nodeRepository->getById($request->byId);
            $activeChildren = count($node->children);

            if ($activeChildren > 0) {
                return response()->json([
                    'error' => 'You can\'t delete, because this node has ' . $activeChildren . ' active children.'
                ], 400);
            }

            DB::beginTransaction();

            $nodeDeleted = $this->nodeRepository->delete($node, Auth::user()->id);

            DB::commit();
    
            return response()->json([
                'message' => 'Node deleted successfully.',
                'data' => $nodeDeleted
            ], 200);
            
        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }
}
