<?php

namespace App\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;

class SaveExpenseRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     */
    public function authorize(): bool
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, \Illuminate\Contracts\Validation\ValidationRule|array<mixed>|string>
     */
    public function rules(): array
    {
        return [
            'id' => 'nullable|uuid|exists:transactions,id',
            'concept' => 'nullable|string|max:255',
            'total_amount' => 'required|numeric|gt:0',
            'date' => 'required|date',
            // Details
            'vault_box_id' => 'required|uuid|exists:vaults_boxes,id',
            'vault_box_history_id' => 'required_with:id|uuid|exists:vault_box_history,id',
            'transaction_method_id' => 'required|uuid|exists:cat_items,id',
            'account_id' => 'nullable|uuid|exists:accounts,id',
            'beneficiary_id' => 'required|uuid|exists:cat_items,id',
            'is_global' => 'required|boolean',
            'nodes' => 'nullable|string',
            'is_billed' => 'nullable|boolean',
            'proof_transaction' => 'nullable|mimes:jpg,jpeg,png,pdf|max:2048',
            'observations' => 'nullable|string',
            // Items
            'items' => 'required|array|min:1',
            'items.*.expense_object_id' => 'required|uuid|exists:expense_objects,id',
            'items.*.quantity' => 'required|integer|min:1'
        ];
    }
}
