<?php

namespace App\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;

class SaveImportExpenseRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     */
    public function authorize(): bool
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, \Illuminate\Contracts\Validation\ValidationRule|array<mixed>|string>
     */
    public function rules(): array
    {
        return [
            'expenses' => 'required|array',
            'expenses.*.concept' => 'required|string|max:255',
            'expenses.*.total_amount' => 'required|numeric|gt:0',
            'expenses.*.date' => 'required|date',
            // Details
            'expenses.*.entity_id' => 'required|uuid|exists:entities,id',
            'expenses.*.vault_box_id' => 'required|uuid|exists:vaults_boxes,id',
            'expenses.*.transaction_method_id' => 'required|uuid|exists:cat_items,id',
            'expenses.*.account_id' => 'nullable|uuid|exists:accounts,id',
            'expenses.*.beneficiary_id' => 'required|uuid|exists:cat_items,id',
            'expenses.*.is_global' => 'required|boolean',
            'expenses.*.nodes' => 'nullable|string',
            'expenses.*.is_billed' => 'nullable|boolean',
            'expenses.*.observations' => 'nullable|string',
            // Items
            'expenses.*.items' => 'required|array|min:1',
            'expenses.*.items.*.expense_object_id' => 'required|uuid|exists:expense_objects,id',
            'expenses.*.items.*.quantity' => 'required|integer|min:1',
            'expenses.*.items.*.single_price' => 'required|numeric|gt:0'
        ];
    }
}
