<?php

namespace App\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;
use App\Models\VaultBox;

class SaveRefundRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     */
    public function authorize(): bool
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, \Illuminate\Contracts\Validation\ValidationRule|array<mixed>|string>
     */
    public function rules(): array
    {
        return [
            'vault_box_id' => [
                'required',
                'uuid',
                'exists:vaults_boxes,id',
                function ($attribute, $value, $fail) {
                    $vaultBox = VaultBox::find($this->input('vault_box_id'));

                    if (is_null($vaultBox->parent)) {
                        $fail('This box has no parent.');
                    }
                }
            ],
            'total_amount' => [
                'required',
                'numeric',
                'gt:0',
                function ($attribute, $value, $fail) {
                    $vaultBox = VaultBox::find($this->input('vault_box_id'));

                    if ($vaultBox->current_amount < $value) {
                        $fail('The ' . $attribute . ' is greater than the current amount in this vault or box.');
                    }
                }
            ],
        ];
    }
}
