<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Concerns\HasUuids;
use Illuminate\Database\Eloquent\SoftDeletes;
use Spatie\Permission\Models\Permission;
use Illuminate\Database\Eloquent\Model;

class CatItem extends Model
{
    use HasUuids, HasFactory, SoftDeletes;

    protected $table = 'cat_items';

    protected $fillable = [
        'catalog_id',
        'catalog_code',
        'name',
        'description',
        'value',
        'created_by',
        'updated_by',
        'deleted_by',
    ];

    protected $hidden = [
        'created_at',
        'updated_at',
        'deleted_at',
        'created_by',
        'updated_by',
        'deleted_by',
    ];

    // Relaciones
    public function catalog()
    {
        return $this->belongsTo(Catalog::class);
    }

    public function permissions()
    {
        return $this->hasMany(Permission::class, 'module_id', 'id');
    }

    public function expenseObjType()
    {
        return $this->hasMany(ExpenseObject::class, 'type_id', 'id');
    }

    public function expenseObjCategory()
    {
        return $this->hasMany(ExpenseObject::class, 'category_id', 'id');
    }

    public function expenseObjGroup()
    {
        return $this->hasMany(ExpenseObject::class, 'group_id', 'id');
    }

    public function expenseObjProvider()
    {
        return $this->hasMany(ExpenseObject::class, 'provider_id', 'id');
    }

    public function nodes()
    {
        return $this->hasMany(Node::class, 'type_id', 'id');
    }

    public function vaultsBoxes()
    {
        return $this->hasMany(VaultBox::class, 'type_id', 'id');
    }

    public function transactionDetails()
    {
        return $this->hasMany(transactionDetail::class, 'bank_id', 'id');
    }
}
