<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Concerns\HasUuids;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Model;

class TransactionDetail extends Model
{
    use HasFactory, HasUuids, SoftDeletes;

    protected $table = 'transaction_details';

    protected $fillable = [
        'transaction_id',
        'entity_id',
        'vault_box_id',
        'vault_box_history_id',
        'dispersed_to',
        'dispersed_history_id',
        'transaction_method_id',
        'bank_id',
        'account_id',
        'beneficiary_id',
        'type',
        'is_global',
        'nodes',
        'is_billed',
        'proof_transaction',
        'observations',
        'created_by',
        'updated_by',
        'deleted_by',
    ];

    protected $hidden = [
        'id',
        'transaction_id',
        'entity_id',
        'vault_box_id',
        'vault_box_history_id',
        'dispersed_to',
        'dispersed_history_id',
        'transaction_method_id',
        'bank_id',
        'account_id',
        'beneficiary_id',
        'created_at',
        'updated_at',
        'deleted_at',
        'created_by',
        'updated_by',
        'deleted_by',
    ];

    // Relaciones
    public function transaction()
    {
        return $this->belongsTo(Transaction::class);
    }

    public function entity()
    {
        return $this->belongsTo(Entity::class);
    }

    public function vaultBox()
    {
        return $this->belongsTo(VaultBox::class);
    }

    public function vaultBoxHistory()
    {
        return $this->belongsTo(VaultBoxHistory::class, 'vault_box_history_id', 'id');
    }

    public function dispersedHistory()
    {
        return $this->belongsTo(VaultBoxHistory::class, 'dispersed_history_id', 'id');
    }

    public function transactionMethod()
    {
        return $this->belongsTo(CatItem::class, 'transaction_method_id', 'id');
    }

    public function bank()
    {
        return $this->belongsTo(CatItem::class, 'bank_id', 'id');
    }

    public function account()
    {
        return $this->belongsTo(Account::class);
    }

    public function beneficiary()
    {
        return $this->belongsTo(CatItem::class, 'beneficiary_id', 'id');
    }

    public function dispersedTo()
    {
        return $this->belongsTo(VaultBox::class, 'dispersed_to', 'id');
    }
}
