<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Concerns\HasUuids;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Model;

class VaultBox extends Model
{
    use HasFactory, HasUuids, SoftDeletes;

    protected $table = 'vaults_boxes';

    protected $fillable = [
        'type_id',
        'entity_id',
        'belongs_to',
        'level',
        'name',
        'current_amount',
        'is_active',
        'status',
        'created_by',
        'updated_by',
        'deleted_by',
    ];

    protected $hidden = [
        'type_id', 
        'entity_id',
        'belongs_to',
        'created_at',
        'updated_at',
        'deleted_at',
        'created_by',
        'updated_by',
        'deleted_by',
    ];

    // Relaciones
    public function type()
    {
        return $this->belongsTo(CatItem::class, 'type_id', 'id');
    }

    public function entity()
    {
        return $this->belongsTo(Entity::class);
    }

    public function children()
    {
        return $this->hasMany(VaultBox::class, 'belongs_to', 'id');
    }

    public function parent()
    {
        return $this->belongsTo(VaultBox::class, 'belongs_to', 'id');
    }

    public function history()
    {
        return $this->hasMany(VaultBoxHistory::class);
    }

    public function users()
    {
        return $this->hasMany(UserVaultBox::class, 'vault_box_id', 'id');
    }

    public function dispersedBy()
    {
        return $this->hasMany(TransactionDetail::class, 'dispersed_to', 'id');
    }  
}
