<?php

namespace App\Repositories;

use App\Models\Account;
use Throwable;

class AccountRepository
{
    public function all()
    {
        try {
            return Account::select('id', 'bank_id', 'entity_id', 'name', 'number')
                ->with(['entity' => function ($entity) {
                        $entity->select('id', 'name');
                }])->with(['bank' => function ($bank) {
                    $bank->select('id', 'name');
                }])->get();

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function save($account)
    {
        try {
            return Account::create([
                'bank_id' => $account->bank_id,
                'entity_id' => $account->entity_id,
                'name' => $account->name,
                'number' => $account->number,
                'created_by' => $account->by_user,
                'updated_by' =>$account->by_user,
            ]);
            
        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function getById($id)
    {
        try {
            return Account::select('id', 'bank_id', 'entity_id', 'name', 'number')
                ->with(['entity' => function ($entity) {
                        $entity->select('id', 'name');
                }])->with(['bank' => function ($bank) {
                    $bank->select('id', 'name');
                }])
                ->find($id);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function update(Account $account, $data)
    {
        try {
            $account->update([
                'bank_id' => $data->bank_id,
                'entity_id' => $data->entity_id,
                'name' => $data->name,
                'number' => $data->number,
                'updated_by' => $data->by_user,
            ]);

            return $account;

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function delete(Account $account, String $byUser)
    {
        try {
            $account->update(['deleted_by' => $byUser]);
            $account->delete();

            return $account;
            
        } catch (Throwable $e) {
            return $e->getMessage();
            
        }
    }
}
