<?php

namespace App\Repositories;

use App\Models\Catalog;
use App\Models\CatItem;
use Throwable;

class CatalogRepository
{
    public function all() // Todos los catálogos
    {
        try {
            return Catalog::select('id', 'name', 'description', 'code', 'color', 'icon')
                ->orderBy('id', 'ASC')
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function save($catalog)
    {
        try {
            return Catalog::create([
                'name' => $catalog['name'],
                'description' => $catalog['description'],
                'code' => $catalog['code'],
                'color' => $catalog['color'],
                'icon' => $catalog['icon'],
                'created_by' => $catalog['by_user'],
                'updated_by' => $catalog['by_user'],
            ]);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getById(String $id)
    {
        try {
            return Catalog::select('id', 'name', 'description', 'code', 'color', 'icon')
                ->find($id);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function update($data)
    {
        try {
            $catalog = $this->getById($data['id']);

            $catalog->update([
                'name' => $data['name'],
                'description' => $data['description'],
                'code' => $data['code'],
                'color' => $data['color'],
                'icon' => $data['icon'],
                'updated_by' => $data['by_user'],
            ]);

            return $catalog;

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
            
        }
    }

    public function delete(String $id, String $byUser)
    {
        try {
            $catalog = $this->getById($id);

            $catalog->update(['deleted_by' => $byUser]);

            $catalog->delete();

            return $catalog;

        } catch (Throwable $e) {
            $e->getMessage();

        }
    }

    public function getItemsByCatalog(String $id)
    {
        try {
            return CatItem::where('catalog_id', $id)
                ->orderBy('id', 'ASC')
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function saveItem($item)
    {
        try {
            return CatItem::create([
                'catalog_id' => $item['catalog_id'],
                'catalog_code' => $item['catalog_code'],
                'name' => $item['name'],
                'description' => $item['description'],
                'value' => $item['value'],
                'created_by' => $item['by_user'],
                'updated_by' => $item['by_user'],
            ]);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getItemById(String $id)
    {
        try {
            return CatItem::select('id', 'catalog_id', 'catalog_code', 'name', 'description', 'value')
                ->find($id);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getItemsByCatCodeAndValue(String $catCode, String $value)
    {
        try {
            return CatItem::select('id', 'catalog_id', 'catalog_code', 'name', 'description', 'value')
                ->where('catalog_code', $catCode)
                ->where('value', $value)
                ->orderBy('name', 'ASC')
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function updateItem($data)
    {
        try {
            $item = $this->getItemById($data['id']);

            $item->update([
                'name' => $data['name'],
                'description' => $data['description'],
                'value' => $data['value'],
                'updated_by' => $data['by_user'],
            ]);

            return $item;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function deleteItem(String $id, String $byUser)
    {
        try {
            $item = CatItem::find($id);

            $item->update(['deleted_by' => $byUser]);

            $item->delete();

            return $item;
            
        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }
}
