<?php

namespace App\Repositories;

use App\Models\Entity;
use Throwable;

class EntityRepository
{
    public function all()
    {
        try {
            return Entity::with(['type' => function ($type) {
                $type->select('id', 'name');
            }])->select('id', 'type_id', 'name', 'is_active', 'status')
                ->orderBy('id', 'ASC')
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function save(Array $entity)
    {
        try {
            return Entity::create([
                'type_id' => $entity['type_id'],
                'name' => $entity['name'],
                'is_active' => TRUE,
                'status' => 'ACTIVO',
                'created_by' => $entity['by_user'],
                'updated_by' => $entity['by_user'],
            ]);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getById(String $id)
    {
        try {
            $entity = Entity::select('id', 'type_id', 'name', 'is_active', 'status')
                ->find($id);

            return $entity;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getByType(String $typeId)
    {
        try {
            return Entity::with(['type' => function ($type) {
                    $type->select('id', 'name') ;
                }])->select('id', 'type_id', 'name', 'is_active', 'status')
                ->where('entity_type_id', $typeId)
                ->orderBy('name', 'ASC')
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function update(Array $data)
    {
        try {
            $entity = $this->getById($data['id']);

            $entity->update([
                'type_id' => $data['type_id'],
                'name' => $data['name'],
                'updated_by' => $data['by_user'],
            ]);

            return $entity;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function delete(String $id, String $byUser)
    {
        try {
            $entity = $this->getById($id);
            
            $entity->update([
                'is_active' => FALSE,
                'status' => 'ELIMINADO',
                'deleted_by' => $byUser,
            ]);
            $entity->delete();

            return $entity;
            
        } catch (Throwable $e) {
            $e->getMessage();

        }
    }
}
