<?php

namespace App\Repositories;

use App\Models\ExpenseObject;
use Throwable;

class ExpenseObjectRepository
{
    public function all()
    {
         try {
            return ExpenseObject::with(['type' => function ($type) {
                    $type->select('id', 'name') ;
                }])->with(['category' => function ($category) {
                    $category->select('id', 'name');
                }])->with(['group' => function ($group) {
                    $group->select('id', 'name');
                }])->with(['provider' => function ($provider) {
                    $provider->select('id', 'name');
                }])->select(    
                    'id', 'type_id', 'category_id', 'group_id', 'provider_id', 
                    'name', 'description', 'status'
                )->orderBy('name', 'ASC')
                ->get();

         } catch (Throwable $e) {
            return $e->getMessage();
         }
    }

    public function save($expenseObject)
    {
        try {
            return ExpenseObject::create([
                'type_id' => $expenseObject['type_id'],
                'category_id' => $expenseObject['category_id'],
                'group_id' => $expenseObject['group_id'],
                'provider_id' => $expenseObject['provider_id'],
                'name' => $expenseObject['name'],
                'description' => $expenseObject['description'],
                'status' => 'ACTIVO',
                'created_by' => $expenseObject['by_user'],
                'updated_by' => $expenseObject['by_user'],
            ]);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getById(String $id)
    {
        try {
            return ExpenseObject::with(['type' => function ($type) {
                    $type->select('id', 'name');
                }])->with(['category' => function ($category) {
                    $category->select('id', 'name');
                }])->with(['group' => function ($group) {
                    $group->select('id', 'name');
                }])->with(['provider' => function ($provider) {
                    $provider->select('id', 'name');
                }])->select(
                    'id', 'type_id', 'category_id', 'group_id', 'provider_id', 
                    'name', 'description', 'status'
                )->orderBy('name', 'ASC')
                ->find($id);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function update(Array $data)
    {
        try {
            $expenseObject = $this->getById($data['id']);
            
            $expenseObject->update([
                'type_id' => $data['type_id'],
                'category_id' => $data['category_id'],
                'group_id' => $data['group_id'],
                'provider_id' => $data['provider_id'],
                'name' => $data['name'],
                'description' => $data['description'],
                'updated_by' => $data['by_user'],
            ]);

            return $expenseObject;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function delete(String $id, String $byUser)
    {
        try {
            $expenseObject = $this->getById($id);

            $expenseObject->update([
                'status' => 'ELIMINADO',
                'deleted_by' => $byUser,
            ]);

            $expenseObject->delete();

            return $expenseObject;

        } catch (Throwable $e) {
            return $e->getMessage();
            
        }
    }
}
