<?php

namespace App\Repositories;

use App\Models\Node;
use Throwable;

class NodeRepository
{
    public function all()
    {
        try {
            return Node::with(['entity' => function ($entity) {
                    $entity->select('id', 'name');
                }])->with(['type' => function ($type) {
                    $type->select('id', 'name');
                }])->with(['parent' => function ($parent) {
                    $parent->select('id', 'name');
                }])->select('id', 'entity_id', 'type_id', 'belongs_to', 
                    'name', 'description', 'is_active', 'status'
                )->get();

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function save($data)
    {
        try {
            $parent = $this->getById($data->belongs_to);

            $node = new Node();
            $node->entity_id = $parent ? $parent->entity_id : $data->entity_id;
            $node->type_id = $parent ? $parent->type_id : $data->type_id;
            $node->belongs_to = $parent ? $parent->id : null;
            $node->name = $data->name;
            $node->description = $data->description;
            $node->is_active = true;
            $node->status = 'ACTIVO';
            $node->created_by = $data->by_user;
            $node->updated_by = $data->by_user;
            $node->save();

            return $node;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getById($id)
    {
        try {
            return Node::with(['entity' => function ($entity) {
                    $entity->select('id', 'name');
                }])->with(['type' => function ($type) {
                    $type->select('id', 'name');
                }])->select('id', 'entity_id', 'type_id', 'name', 
                    'description', 'is_active', 'status', 'belongs_to'
                )->find($id);
            
        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function update($data)
    {
        try {
            $node = $this->getById($data->id);
        
            $node->update([
                'entity_id' => $data->entity_id,
                'type_id' => $data->type_id,
                'belongs_to' => $data->belongs_to,
                'name' => $data->name,
                'description' => $data->description,
                'updated_by' => $data->by_user,
            ]);

            return $node;            

        } catch (Throwable $e) {
            return $e->getMessage();
            
        }
    }

    public function delete(Node $node, String $byUser)
    {
        try {
            $node->update([
                'is_active' => false,
                'status' => 'ELIMINADO',
                'deleted_by' => $byUser
            ]);

            $node->delete();

            return $node;
            
        } catch (Throwable $e) {
            return $e->getMessage();
        } 
    }
}
