<?php

namespace App\Repositories;

use Spatie\Permission\Models\Permission;
use App\Models\Catalog;
use Throwable;

class PermissionRepository
{
    public function all()
    {
        return Permission::with(['module' => function ($module) {
                $module->select('id', 'name');
            }])->orderBy('name', 'ASC')
            ->get();
    }

    public function save(Array $permission)
    {
        try {
            return Permission::create([
                'name' => $permission['name'],
                'tag' => $permission['tag'],
                'module_id' => $permission['module_id'],
            ]);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function update(Array $data)
    {
        try {
            $permission = $this->getById($data['id']);

            $permission->update([
                'name' => $data['name'],
                'tag' => $data['tag'],
                'module_id' => $data['module_id'],
            ]);

            return $permission;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getById(String $id)
    {
        try {
            return Permission::select('id', 'name', 'tag', 'module_id')
                ->findOrFail($id);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function delete(String $id)
    {
        try {
            $permission = $this->getById($id);

            $permission->delete();

            return $permission;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function byModules()
    {
        try {
            return Catalog::with(['items' => function ($items) {
                    $items->with(['permissions' => function ($permissions) {
                        $permissions->select('id', 'name', 'tag', 'module_id');
                    }])->select('id', 'catalog_id', 'name');
                }])->where('code', 'CAT_MODULES')
                ->select('id', 'name', 'code')
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }
}
