<?php

namespace App\Repositories;

use App\Models\Person;
use Throwable;

class PersonRepository
{
    public function all()
    {
        try {
            return Person::select('id', 'name', 'last_name', 'is_active', 'status')
                ->orderBy('id', 'ASC')
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function save(Array $person)
    {
        try {
            return Person::create([
                'name' => $person['name'],
                'last_name' => $person['last_name'],
                'is_active' => TRUE,
                'status' => 'ACTIVO',
                'created_by' => $person['by_user'],
                'updated_by' => $person['by_user'],
            ]);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getById(String $id)
    {
        try {
            return Person::select('id', 'name', 'last_name', 'is_active', 'status')
                ->find($id);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function searchByName(String $name)
    {
        try {
            return Person::select('id', 'name', 'last_name', 'is_active', 'status')
                ->where('name', 'like', '%' . $name . '%')
                ->orWhere('last_name', 'like', '%' . $name . '%')
                ->orderBy('id', 'ASC')
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function update(Array $data)
    {
        try {
            $person = $this->getById($data['id']);

            $person->update([
                'name' => $data['name'],
                'last_name' => $data['last_name'],
                'updated_by' => $data['by_user'],
            ]);

            return $person;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function delete(String $id, String $byUser)
    {
        try {
            $person = $this->getById($id);

            $person->update([
                'is_active' => false,
                'status' => 'ELIMINADO',
                'deleted_by' => $byUser,
            ]);

            $person->delete();

            return $person;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }
}
