<?php

namespace App\Repositories;

use Spatie\Permission\Models\Role;
use Throwable;

class RoleRepository
{
    public function all()
    {
        try {
            return Role::all();
            
        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function save(Array $role)
    {
        try {
            return Role::create([
                'name' => $role['name'],
                'color' => $role['color'],
                'icon' => $role['icon'],
            ]);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getById(String $id)
    {
        try {
            return Role::with(['permissions' => function ($permissions) {
                    $permissions->select('id', 'name', 'tag', 'module_id')
                        ->orderBy('name', 'ASC');
                }])->select('id', 'name', 'color', 'icon')
                ->find($id);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function update(Array $data)
    {
        try {
            $role = $this->getById($data['id']);

            $role->update([
                'name' => $data['name'],
                'color' => $data['color'],
                'icon' => $data['icon'],
            ]);

            return $role;
            
        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function delete(String $id)
    {
        try {
            $role = $this->getById($id);

            $role->delete();

            return $role;

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }
}
