<?php

namespace App\Repositories;

use App\Models\TransactionDetail;
use Throwable;

class TransactionDetailRepository
{
    public function save($transactionDetail)
    {
        try {
            return TransactionDetail::create([
                'transaction_id' => $transactionDetail['transaction_id'],
                'entity_id' => $transactionDetail['entity_id'],
                'vault_box_id' => $transactionDetail['vault_box_id'] ?? null,
                'vault_box_history_id' => $transactionDetail['vault_box_history_id'] ?? null,
                'dispersed_to' => $transactionDetail['dispersed_to'] ?? null,
                'dispersed_history_id' => $transactionDetail['dispersed_history_id'] ?? null,
                'transaction_method_id' => $transactionDetail['transaction_method_id'] ?? null,
                'bank_id' => $transactionDetail['bank_id'] ?? null,
                'account_id' => $transactionDetail['account_id'] ?? null,
                'beneficiary_id' => $transactionDetail['beneficiary_id'] ?? null,
                'type' => $transactionDetail['type'],
                'is_global' => $transactionDetail['is_global'] ?? null,
                'nodes' => $transactionDetail['nodes'] ?? null,
                'is_billed' => $transactionDetail['is_billed'] ?? null,
                'observations' => $transactionDetail['observations'] ?? null,
                'created_by' => $transactionDetail['by_user'],
                'updated_by' => $transactionDetail['by_user'],
            ]);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getById(String $id)
    {
        try {
            return TransactionDetail::select(
                'id', 'transaction_id', 'entity_id', 'vault_box_id', 
                'transaction_method_id', 'bank_id', 'account_id',
                'beneficiary_id', 'dispersed_to', 'type', 'is_global', 
                'nodes', 'is_billed', 'proof_transaction', 'observations'
            )->find($id);

        } catch (Throwable $e) {
            $e->getMessage();

        }
    }

    public function update($transactionDetail, $data) {
        try {
            $transactionDetail->update([
                'entity_id' => $data['entity_id'] ?? $transactionDetail->entity_id,
                'vault_box_id' => $data['vault_box_id'] ?? $transactionDetail->vault_box_id,
                'vault_box_history_id' => $data['vault_box_history_id'] ?? $transactionDetail->vault_box_history_id,
                'transaction_method_id' => $data['transaction_method_id'] ?? $transactionDetail->transaction_method_id,
                'bank_id' => $data['bank_id'] ?? $transactionDetail->bank_id,
                'account_id' => $data['account_id'] ?? $transactionDetail->account_id,
                'beneficiary_id' => $data['beneficiary_id'] ?? $transactionDetail->beneficiary_id,
                'is_global' => $data['is_global'] ?? $transactionDetail->is_global,
                'nodes' => $data['nodes'] ?? $transactionDetail->nodes,
                'is_billed' => $data['is_billed'] ?? $transactionDetail->is_billed,
                'observations' => $data['observations'] ?? $transactionDetail->observations,
                'updated_by' => $data['by_user'],
            ]);

            return $transactionDetail;

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function uploadProofTransaction(Array $data)
    {
        try {
            $transactionDetail = $this->getById($data['detail_id']);

            $path = $data['proof_transaction']->store('private/transaction_vouchers');
            
            $transactionDetail->update([
                'proof_transaction' => $path,
                'updated_by' => $data['by_user'],
            ]);

            return $transactionDetail;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getProofTransaction(String $id)
    {
        try {
            return TransactionDetail::select('proof_transaction')->find($id);

        } catch (Throwable $e) {
            return $e->getMessage();
            
        }
    }

    public function deleteProofTransaction(Array $data)
    {
        try {
            $transactionDetail = $this->getById($data->transaction_detail_id);
            $transactionDetail->update([
                'proof_transaction' => null,
                'updated_by' => $data->by_user,
            ]);

            return $transactionDetail;
            
        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }
}
