<?php

namespace App\Repositories;

use App\Models\TransactionItem;
use Throwable;


class TransactionItemRepository
{
    public function getAllOfTransaction(String $transactionId)
    {
        try {
            return TransactionItem::where('transaction_id', $transactionId)
                ->with(['expenseObject' => function ($expenseObject) {
                    $expenseObject->select('id', 'type_id', 'category_id', 'group_id', 'name')
                    ->with(['category' => function ($category) {
                        $category->select('id', 'name');
                    }])->with(['group' => function ($group) {
                        $group->select('id', 'name');
                    }])->with(['type' => function ($type) {
                        $type->select('id', 'name');
                    }]);
                }])->get();

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function save(Array $data)
    {
        try {
            foreach ($data['items'] as $item) {
                TransactionItem::create([
                    'transaction_id' => $data['transaction_id'],
                    'expense_object_id' => $item['expense_object_id'],
                    'quantity' => $item['quantity'],
                    'single_price' => $item['single_price'],
                    'created_by' => $data['by_user'],
                    'updated_by' => $data['by_user'],
                ]);
            }

            return TransactionItem::where('transaction_id', $data['transaction_id'])
                ->select('id', 'transaction_id', 'expense_object_id', 'quantity', 'single_price')
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function update(Array $data)
    {
        try {
            $items = TransactionItem::where('transaction_id', $data['transaction_id'])->get();
            $existingItems = [];
            $itemsSent = [];

            foreach ($items as $item) { // Para obtener los id's de los items que ya pertenecen al gasto
                array_push($existingItems, $item->expense_object_id);
            }

            foreach ($data['items'] as $itemSent) { // Para obtener los id's de los items que se envían y agregar los nuevos
                array_push($itemsSent, $itemSent['expense_object_id']);
                
                if (in_array($itemSent['expense_object_id'], $existingItems)) {
                    $itemToUpdate = TransactionItem::where('transaction_id', $data['transaction_id'])
                        ->where('expense_object_id', $itemSent['expense_object_id'])->first();
                    $itemToUpdate->update([
                        'quantity' => $itemSent['quantity'],
                        'single_price' => $itemSent['single_price'],
                        'updated_by' => $data['by_user'],
                    ]);

                } else {
                    TransactionItem::create([
                        'transaction_id' => $data['transaction_id'],
                        'expense_object_id' => $itemSent['expense_object_id'],
                        'quantity' => $itemSent['quantity'],
                        'single_price' => $itemSent['single_price'],
                        'created_by' => $data['by_user'],
                        'updated_by' => $data['by_user'],
                    ]);
                }
            }

            foreach ($existingItems as $existingItem) { // Para remover los items que ya no estarán
                if (!in_array($existingItem, $itemsSent)) {
                    $itemToDelete = TransactionItem::where('transaction_id', $data['transaction_id'])
                        ->where('expense_object_id', $existingItem)->first();
                    $itemToDelete->update(['deleted_by' => $data['by_user']]);
                    $itemToDelete->delete();
                }
            }

            return TransactionItem::where('transaction_id', $data['transaction_id'])->get();
        
        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }
}
