<?php

namespace App\Repositories;

use App\Models\Transaction;
use Throwable;

class TransactionRepository
{
    public function all()
    {
        try {
            return Transaction::select('id', 'concept', 'total_amount', 'date', 'comment')
                ->with(['details' => function ($details) {
                    $details->with(['entity' => function ($entity) {
                        $entity->select('id', 'name');
                    }])->with(['vaultBox' => function ($vaultBox) {
                        $vaultBox->select('id', 'name');
                    }])->with(['transactionMethod' => function ($paymentMethod) {
                        $paymentMethod->select('id', 'name');
                    }])->with(['bank' => function ($bank) {
                        $bank->select('id', 'name');
                    }])->with(['account' => function ($account) {
                        $account->select('id', 'name');
                    }])->with(['beneficiary' => function ($beneficiary) {
                        $beneficiary->select('id', 'name');
                    }])->with(['dispersedTo' => function ($dispersedTo) {
                        $dispersedTo->select('id', 'name');
                    }]);
                }])->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function save($transaction)
    {
        try {
            return Transaction::create([
                'concept' => $transaction['concept'],
                'total_amount' => $transaction['total_amount'],
                'date' => $transaction['date'],
                'created_by' => $transaction['by_user'],
                'updated_by' => $transaction['by_user'],
            ]);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getById(String $id)
    {
        try {
            return Transaction::with('details')
                ->select('id', 'total_amount', 'date', 'concept')
                ->find($id);

        } catch (Throwable $e) {
            $e->getMessage();
        }
    }

    public function getByVaultBoxAndDate($query)
    {
        try {
            return Transaction::select('id', 'concept', 'total_amount', 'date', 'comment')
                ->whereBetween('date', [$query->startDate, $query->endDate])
                ->whereHas('details', function ($details) use ($query) {
                    $details->where('vault_box_id', $query->vaultBoxId)
                        ->orWhere('dispersed_to', $query->vaultBoxId);
                })->with(['details' => function ($details) {
                    $details->with(['entity' => function ($entity) {
                        $entity->select('id', 'name');
                    }])->with(['vaultBox' => function ($vaultBox) {
                        $vaultBox->select('id', 'name');
                    }])->with(['transactionMethod' => function ($paymentMethod) {
                        $paymentMethod->select('id', 'name');
                    }])->with(['bank' => function ($bank) {
                        $bank->select('id', 'name');
                    }])->with(['account' => function ($account) {
                        $account->select('id', 'name');
                    }])->with(['beneficiary' => function ($beneficiary) {
                        $beneficiary->select('id', 'name');
                    }])->with(['dispersedTo' => function ($dispersedTo) {
                        $dispersedTo->select('id', 'name');
                    }]);
                }])->orderBy('date', 'ASC')
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function update($data)
    {
        try {
            $transaction = $this->getById($data['id']);

            $transaction->update([
                'concept' => $data['concept'] ?? $transaction->concept,
                'total_amount' => $data['total_amount'] ?? $transaction->total_amount,
                'date' => $data['date'] ?? $transaction->date,
                'comment' => $data['comment'],
                'update_by' => $data['by_user'],
            ]);

            return $transaction;
            
        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function delete($data)
    {
        try {
            $transaction = $this->getById($data['id']);

            $transaction->update([
                'comment' => $data['comment'],
                'deleted_by' => $data['by_user'],
            ]);

            $transaction->delete();

            return $transaction;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }
}
