<?php

namespace App\Repositories;

use Illuminate\Support\Facades\Hash;
use App\Models\User;
use Throwable;

class UserRepository
{
    public function all()
    {
        try {
            return User::with(['person' => function ($person) {
                    $person->select('id', 'name', 'last_name', 'is_active', 'status');
                }])->with('roles')
                ->select('id', 'person_id', 'email', 'is_active', 'status', 'is_system_admin')
                ->orderBy('email', 'ASC')
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getById(String $id)
    {
        try {
            return User::with(['roles' => function ($roles) {
                    $roles->select('name');
                }])->with(['person' => function ($person) {
                    $person->select('id', 'name', 'last_name', 'is_active', 'status');
                }])
                ->select('id', 'person_id', 'email', 'profile_photo', 'is_active', 'status', 'is_system_admin')
                ->find($id);

        } catch (Throwable $e) {
            $e->getMessage();
            
        }
    }

    public function getByEmail(String $email)
    {
        try {
            return User::with(['roles' => function ($roles) {
                    $roles->with('permissions');
                }])->with(['person' => function ($person) {
                    $person->select('id', 'name', 'last_name', 'is_active', 'status');
                }])
                ->select('id', 'person_id', 'email', 'password', 'profile_photo', 
                    'last_password_update', 'is_active', 'status', 'is_system_admin'
                )
                ->where('email', $email)->first();

        } catch (Throwable $e) {
            return $e->getMessage();
            
        }
    }

    public function save(Array $user)
    {
        try {
            return User::create([
                'person_id' => $user['person_id'],
                'email' => $user['email'],
                'password' => Hash::make($user['password']),
                'is_active' => TRUE,
                'status' => 'ACTIVO',
                'is_system_admin' => $user['is_system_admin'],
                'created_by' => $user['by_user'],
                'updated_by' => $user['by_user'],
            ]);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function update(User $user, Array $data, String $byUser)
    {
        try {
            $user->update([
                'email' => $data['email'],
                'is_system_admin' => $data['is_system_admin'],
                'updated_by' => $byUser,
            ]);

            return $user;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function changePassword(Array $data)
    {
        try {
            $user = User::findOrFail($data['id']);

            return $user->update([
                'password' => Hash::make($data['password']),
                'updated_by' => $data['by_user'],
                'last_password_update' => Date(now()),
            ]);

        } catch (Throwable $e) {
            $e->getMessage();

        }
    }

    public function toggleStatus(String $id, String $byUser)
    {
        try {
            $user = $this->getById($id);

            $user->is_active = !$user->is_active;
            $user->updated_by = $byUser;

            if ($user->status == 'ACTIVO') {
                $user->status = 'INACTIVO';
            } else {
                $user->status = 'ACTIVO';
            }

            $user->save();

            return $user;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function uploadProfilePhoto(Array $data)
    {
        try {
            $user = $this->getById($data['user_id']);
            $path = $data['profile_picture']->store('private/img/profile_pictures');
            $user->profile_photo = $path;

            return $user->save();

        } catch (Throwable $e) {
            return $e->getMessage();
            
        }
    }

    public function getProfilePhoto(String $id)
    {
        try {
            return User::select('profile_photo')->find($id);
            
        } catch (Throwable $e) {
            return $e->getMessage();
            
        }
    }

    public function delete(String $id)
    {
        // 
    }
}
