<?php

namespace App\Repositories;

use App\Models\UserVaultBox;
use App\Models\User;
use Throwable;

class UserVaultBoxRepository
{
    public function save($userVaultBox)
    {
        try {
            return UserVaultBox::create([
                'user_id' => $userVaultBox->user_id,
                'vault_box_id' => $userVaultBox->vault_box_id,
                'created_by' => $userVaultBox->by_user,
                'updated_by' => $userVaultBox->by_user,
            ]);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getOne(String $userId, String $vaultBoxId)
    {
        try {
            return UserVaultBox::where('user_id', $userId)
                ->where('vault_box_id', $vaultBoxId)
                ->first();
            
        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function getManagersByVaultBox(String $vaultBoxId)
    {
        try {
            return UserVaultBox::select('id', 'user_id', 'vault_box_id')
                ->where('vault_box_id', $vaultBoxId)
                ->with(['users' => function ($users) {
                    $users->with(['person' => function ($person) {
                        $person->select('id', 'name', 'last_name');
                    }])->select('id', 'person_id', 'email', 'is_active', 'status');
                }])
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function delete(UserVaultBox $userVaultBox, String $byUser)
    {
        try {
            $userVaultBox->update(['deleted_by' => $byUser]);
            $userVaultBox->delete();

            return $userVaultBox;

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    } 
}
