<?php

namespace App\Repositories;

use App\Models\VaultBoxHistory;
use Throwable;

class VaultBoxHistoryRepository
{
    public function getAllOfOne(String $vaultBoxId)
    {
        try {
            return VaultBoxHistory::where('vault_box_id', $vaultBoxId)
                ->orderBy('start_date', 'DESC')
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function getById($id)
    {
        try {
            return VaultBoxHistory::find($id);

        } catch (Throwable $e) {
            return $e->getMessage();
            
        }
    }

    public function getOneOpen(String $vaultBoxId, String $date, String $order)
    {
        try {
            return VaultBoxHistory::where('vault_box_id', $vaultBoxId)
                ->whereDate('start_date', '<=', $date)
                ->whereNull('end_date')
                ->orderBy('start_date', $order)
                ->first();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getByDate(String $vaultBoxId, String $date)
    {
        try {
            return VaultBoxHistory::where('vault_box_id', $vaultBoxId)
                ->whereDate('start_date', '<=', $date)
                ->where(function ($history) use ($vaultBoxId, $date) {
                    $history->whereDate('end_date', '>=', $date)
                        ->orWhere(function ($history) use ($vaultBoxId) {
                            $history->whereNull('end_date')
                                ->where('vault_box_id', $vaultBoxId);
                        });
                })->first();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getByOneDate(String $vaultBoxId, String $date)
    {
        try {
            return VaultBoxHistory::where('vault_box_id', $vaultBoxId)
                ->whereDate('start_date', $date)
                ->whereDate('end_date', $date)
                ->first();

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function getOfOne(String $vaultBoxId, String $date)
    {
        try {
            return VaultBoxHistory::where('vault_box_id', $vaultBoxId)
                ->whereDate('start_date', '<=', $date)
                ->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function save($history)
    {
        try {
            return VaultBoxHistory::create([
                'vault_box_id' => $history['vault_box_id'],
                'start_date' => $history['start_date'],
                'end_date' => $history['end_date'] ?? null,
                'initial_balance' => $history['initial_balance'],
                'final_balance' => $history['final_balance'] ?? null,
                'total_income' => $history['total_income'],
                'total_expenditure' => $history['total_expenditure'],
                'total_spent' => $history['total_spent'],
                'created_by' => $history['by_user'],
                'updated_by' => $history['by_user'],
            ]);

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function update($data)
    {
        // 
    }

    public function addIncome(VaultBoxHistory $vaultBoxHistory, $data)
    {
        try {
            $vaultBoxHistory->total_income += $data['amount'];
            $vaultBoxHistory->updated_by = $data['by_user'];
            $vaultBoxHistory->save();

            return $vaultBoxHistory;

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function addEgress(VaultBoxHistory $vaultBoxHistory, $data)
    {
        try {
            $vaultBoxHistory->total_expenditure += $data['amount'];
            $vaultBoxHistory->updated_by = $data['by_user'];
            $vaultBoxHistory->save();

            return $vaultBoxHistory;

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function addExpense(VaultBoxHistory $vaultBoxHistory, $data)
    {
        try {
            if (!is_null($vaultBoxHistory->final_balance)) {
                $vaultBoxHistory->final_balance -= $data['amount'];
            }

            $vaultBoxHistory->total_spent += $data['amount'];
            $vaultBoxHistory->updated_by = $data['by_user'];
            $vaultBoxHistory->save();

            return $vaultBoxHistory;
            
        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function returnEgress(VaultBoxHistory $vaultBoxHistory, $data)
    {
        try {
            
            if (!is_null($vaultBoxHistory->final_balance)) {
                $vaultBoxHistory->final_balance += $data['amount'];
            }
            $vaultBoxHistory->total_spent -= $data['amount'];
            $vaultBoxHistory->updated_by = $data['by_user'];
            $vaultBoxHistory->save();

            return $vaultBoxHistory;

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function returnDispersion(VaultBoxHistory $historyParent, VaultBoxHistory $historyChild, $data)
    {
        try {
            $history = array();

            // Parent
            if ($historyParent->final_balance != 0) {
                $historyParent->final_balance += $data['amount'];
            }
            $historyParent->total_expenditure -= $data['amount'];
            $historyParent->updated_by = $data['by_user'];
            $historyParent->save();

            array_push($history, $historyParent);

            // Child
            if ($historyChild->final_balance != 0) {
                $historyChild->final_balance -= $data['amount'];
            }
            $historyChild->total_income -= $data['amount'];
            $historyChild->updated_by = $data['by_user'];
            $historyChild->save();

            array_push($historyChild, $history);

            return $history;


        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function close(VaultBoxHistory $vaultBoxHistory, Array $data)
    {
        try {
            $vaultBoxHistory->update([
                'end_date' => $data['end_date'],
                'final_balance' => $data['final_balance'],
                'updated_by' => $data['by_user'],
            ]);

            return $vaultBoxHistory;

        } catch (Throwable $e) {
            return $e->getMessage();
        }   
    }
}
