<?php

namespace App\Repositories;

use App\Models\VaultBox;
use App\Models\UserVaultBox;
use Throwable;

class VaultBoxRepository
{
    public function all()
    {
        try {
            return VaultBox::with(['type' => function ($type) {
                    $type->select('id', 'name');
                }])->with(['entity' => function ($entity) {
                    $entity->select('id', 'name');
                }])->with(['parent' => function ($parent) {
                    $parent->select('id');
                }])->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function getAllByUser(String $userId)
    {
        try {
            return VaultBox::whereHas('users', function ($users) use ($userId) {
                    $users->where('user_id', $userId);
                })->with(['type' => function ($type) {
                    $type->select('id', 'name');
                }])->with(['entity' => function ($entity) {
                    $entity->select('id', 'name');
                }])->with(['parent' => function ($parent) {
                    $parent->select('id');
                }])->get();

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function save($vaultBox)
    {
        try {
            return VaultBox::create([
                'type_id' => $vaultBox->type_id,
                'entity_id' => $vaultBox->entity_id,
                'belongs_to' => $vaultBox->belongs_to,
                'level' => $vaultBox->level,
                'name' => $vaultBox->name,
                'current_amount' => 0,
                'is_active' => TRUE,
                'status' => 'ACTIVO',
                'created_by' => $vaultBox->by_user,
                'updated_by' => $vaultBox->by_user,
            ]);

        } catch (Throwable $e) {
            return $e->getMessage();
            
        }
    }

    public function getById($id)
    {
        try {
            return VaultBox::find($id);

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function update($vaultBox, $data)
    {
        try {
            $vaultBox->update([
                'name' => $data->name,
                'updated_by' => $data->by_user,
            ]);

            return $vaultBox;

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function addIncome(VaultBox $vaultBox, $data)
    {
        try {
            $vaultBox->current_amount += $data['amount'];
            $vaultBox->updated_by = $data['by_user'];
            $vaultBox->save();

            return $vaultBox;

        } catch (Throwable $e) {
            return $e->getMessage();
            
        }
    }

    public function toggleStatus(VaultBox $vaultBox, String $byUser)
    {
        try {
            $vaultBox->is_active = !$vaultBox->is_active;
            $vaultBox->updated_by = $byUser;

            if ($vaultBox->status == 'ACTIVO') {
                $vaultBox->status = 'INACTIVO';
            } else {
                $vaultBox->status = 'ACTIVO';
            }

            $vaultBox->save();

            return $vaultBox;

        } catch (Throwable $e) {
            return $e->getMessage();

        }
    }

    public function delete(VaultBox $vaultBox, String $byUser)
    {
        try {
            $vaultBox->update([
                'is_active' => false,
                'status' => 'ELIMINADO',
                'deleted_by' => $byUser,
            ]);

            $vaultBox->delete();

            return $vaultBox;

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function addCurrentAmount(VaultBox $vaultBox, $data)
    {
        try {
            $vaultBox->current_amount += $data['amount'];
            $vaultBox->updated_by = $data['by_user'];
            $vaultBox->save();

            return $vaultBox;

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function subtractCurrentAmount(VaultBox $vaultBox, $data)
    {
        try {
            $vaultBox->current_amount -= $data['amount'];
            $vaultBox->updated_by = $data['by_user'];
            $vaultBox->save();

            return $vaultBox;

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }
}
