<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    /**
     * Run the migrations.
     */
    public function up(): void
    {
        Schema::create('vaults_boxes', function (Blueprint $table) {
            $table->uuid('id')->primary();
            $table->uuid('type_id');
            $table->uuid('entity_id');
            $table->uuid('belongs_to')->nullable()->default(null);
            $table->integer('level')->unsigned();
            $table->string('name', 255);
            $table->double('current_amount', 15, 8);
            $table->boolean('is_active')->default(true);
            $table->string('status', 25);

            $table->timestamps();
            $table->softDeletes();

            $table->uuid('created_by');
            $table->uuid('updated_by');
            $table->uuid('deleted_by')->nullable()->default(null);

            $table->foreign('type_id')->references('id')->on('cat_items')->onDelete('cascade');
            $table->foreign('entity_id')->references('id')->on('entities')->onDelete('cascade');
            $table->foreign('belongs_to')->references('id')->on('vaults_boxes')->onDelete('cascade');
        });

        Schema::create('vault_box_history', function (Blueprint $table) {
            $table->uuid('id')->primary();
            $table->uuid('vault_box_id');
            $table->date('start_date');
            $table->date('end_date')->nullable()->default(null);
            $table->double('initial_balance', 15, 8);
            $table->double('final_balance', 15, 8);
            $table->double('total_income', 15, 8);
            $table->double('total_expenditure', 15, 8);
            $table->double('total_spent', 15, 8);
            
            $table->timestamps();
            $table->softDeletes();

            $table->uuid('created_by');
            $table->uuid('updated_by');
            $table->uuid('deleted_by')->nullable()->default(null);

            $table->foreign('vault_box_id')->references('id')->on('vaults_boxes')->onDelete('cascade');
        });

        Schema::create('users_vaults_boxes', function (Blueprint $table) {
            $table->uuid('id')->primary();
            $table->uuid('user_id');
            $table->uuid('vault_box_id');

            $table->timestamps();
            $table->softDeletes();

            $table->uuid('created_by');
            $table->uuid('updated_by');
            $table->uuid('deleted_by')->nullable()->default(null);

            $table->foreign('user_id')->references('id')->on('users')->onDelete('cascade');
            $table->foreign('vault_box_id')->references('id')->on('vaults_boxes')->onDelete('cascade');
        });
    }

    /**
     * Reverse the migrations.
     */
    public function down(): void
    {
        Schema::dropIfExists('vaults_boxes');
        Schema::dropIfExists('vault_box_history');
        Schema::dropIfExists('users_vaults_boxes');
    }
};
