<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    /**
     * Run the migrations.
     */
    public function up(): void
    {
        Schema::create('transactions', function (Blueprint $table) {
            $table->uuid('id')->primary();
            $table->string('concept', 150)->nullable();
            $table->double('total_amount', 15, 8);
            $table->timestamp('date')->default(now());
            $table->text('comment')->nullable();
            
            $table->timestamps();
            $table->softDeletes();

            $table->uuid('created_by');
            $table->uuid('updated_by');
            $table->uuid('deleted_by')->nullable()->default(null);
        });

        Schema::create('transaction_details', function (Blueprint $table) {
            $table->uuid('id')->primary();
            $table->uuid('transaction_id');
            $table->uuid('entity_id');
            $table->uuid('vault_box_id');
            $table->uuid('vault_box_history_id');
            $table->uuid('dispersed_to')->nullable()->default(null);
            $table->uuid('dispersed_history_id')->nullable()->default(null);
            $table->uuid('transaction_method_id')->nullable();
            $table->uuid('bank_id')->nullable()->default(null);
            $table->uuid('account_id')->nullable()->default(null);
            $table->uuid('beneficiary_id')->nullable()->default(null);
            $table->enum('type', ['E', 'D', 'I', 'R']); // E: expense, D: dispersion, I: income, R: refund
            $table->boolean('is_global')->nullable()->default(null);
            $table->json('nodes')->nullable();
            $table->boolean('is_billed')->nullable()->default(null);
            $table->string('proof_transaction')->nullable();
            $table->text('observations')->nullable();
            
            $table->timestamps();
            $table->softDeletes();

            $table->uuid('created_by');
            $table->uuid('updated_by');
            $table->uuid('deleted_by')->nullable()->default(null);

            $table->foreign('transaction_id')->references('id')->on('transactions')->onDelete('cascade');
            $table->foreign('entity_id')->references('id')->on('entities')->onDelete('cascade');
            $table->foreign('vault_box_id')->references('id')->on('vaults_boxes')->onDelete('cascade');
            $table->foreign('vault_box_history_id')->references('id')->on('vault_box_history')->onDelete('cascade');
            $table->foreign('dispersed_to')->references('id')->on('vaults_boxes')->onDelete('cascade');
            $table->foreign('dispersed_history_id')->references('id')->on('vault_box_history')->onDelete('cascade');
            $table->foreign('transaction_method_id')->references('id')->on('cat_items')->onDelete('cascade');
            $table->foreign('bank_id')->references('id')->on('cat_items')->onDelete('cascade');
            $table->foreign('account_id')->references('id')->on('accounts')->onDelete('cascade');
            $table->foreign('beneficiary_id')->references('id')->on('cat_items')->onDelete('cascade');
        });

        Schema::create('transaction_items', function (Blueprint $table) {
            $table->uuid('id')->primary();
            $table->uuid('transaction_id');
            $table->uuid('expense_object_id');
            $table->integer('quantity')->unsigned();
            $table->double('single_price', 15, 8);
            
            $table->timestamps();
            $table->softDeletes();

            $table->uuid('created_by');
            $table->uuid('updated_by');
            $table->uuid('deleted_by')->nullable()->default(null);

            $table->foreign('transaction_id')->references('id')->on('transactions')->onDelete('cascade');
            $table->foreign('expense_object_id')->references('id')->on('expense_objects')->onDelete('cascade');
        });
    } 

    /**
     * Reverse the migrations.
     */
    public function down(): void
    {
        Schema::dropIfExists('transactions');
        Schema::dropIfExists('transaction_details');
        Schema::dropIfExists('transaction_items');
    }
};
