<?php

namespace Database\Seeders;

use Illuminate\Database\Console\Seeds\WithoutModelEvents;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\Seeder;
use App\Models\Catalogs;
use App\Models\CatItem;

class CatItemSeeder extends Seeder
{
    /**
     * Run the database seeds.
     */
    public function run(): void
    {
        // Módulos
        $this->addItems('CAT_MODULES', array(
            'INICIO',
            'TRANSACCIONES',
            'GASTOS',
            'OBJETOS DE GASTO',
            'ENTIDADES',
            'BÓVEDAS Y CAJAS',
            'CATÁLOGOS',
            'PERSONAS',
            'USUARIOS',
            'ROLES',
            'PERMISOS',
        ));

        // Tipos de entidades
        $this->addItems('CAT_ENTITY_TYPES', array(
            'PERSONA',
            'EMPRESA',
            'ORGANIZACIÓN',
        ));

        // Categorías de Objetos de Gastos
        $this->addItems('CAT_EXPEND_OBJ_CATEGORIES', array(
            'PRODUCTO',
            'SERVICIO',
            'ACTIVO',
        ));

        // Métodos de pago
        $this->addItems('CAT_PAYMENT_METHODS', array(
            'TRANSFERENCIA',
            'EFECTIVO',
        ));

        // Tipos de contenedores
        $this->addItems('CAT_CONTAINER_TYPES', array(
            'BÓVEDA',
            'CAJA'
        ));
    }

    public function addItems(String $code, Array $itemNames)
    {
        try {
            DB::beginTransaction();

            $catalog = Catalog::firstWhere('code', $code);
            
            foreach ($itemNames as $itemName) {
                CatItem::create([
                    'catalog_id' => $catalog->id,
                    'catalog_code' => $catalog->code,
                    'name' => $itemName,
                ]);
            }

            DB::commit();

        } catch (Throwable $e) {
            DB::rollback();
            
            return $e->getMessage();
        }
    }
}
