<?php

namespace Database\Seeders;

use Illuminate\Database\Console\Seeds\WithoutModelEvents;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\Seeder;
use Spatie\Permission\Models\Permission;
use App\Modules\CatItem;

class PermissionSeeder extends Seeder
{
    /**
     * Run the database seeds.
     */
    public function run(): void
    {
        // Objetos de Gastos
        $this->addPermissions('OBJETOS DE GASTO', array(
            'expense-objects|Ver objetos',
            'expense-objects.create|Registrar objetos',
            'expense-objects.get|Obtener objetos',
            'expense-objects.edit|Editar objetos',
            'expense-objects.delete|Eliminar objetos',
        ));

        // Transacciones
        $this->addPermissions('TRANSACCIONES', array(
            'transactions|Ver transacciones',
            'transactions.upload-proof-payment|Cargar comprobante',
            'transactions.get-proof-payment|Cargar comprobante',
            // 'transactions.expenses.register|Registrar gastos',
            // 'expenses.get|Obtener gastos',
            // 'expenses.edit|Editar gastos',
            // 'expenses.details|Detalles de gastos',
            // 'expenses.delete|Eliminar gastos',
            // 'expenses.details.save|Guardar detalles',
            'transactions.items.save|Guardar items',
        ));

        // Entidades
        $this->addPermissions('ENTIDADES', array(
            'entities|Ver entidades',
            'entities.create|Registrar entidades',
            'entities.get|Obtener entidades',
            'entities.edit|Editar entidades',
            'entities.delete|Eliminar entidades',
        ));

        // Personas
        $this->addPermissions('PERSONAS', array(
            'people|Ver personas',
            'people.create|Registrar personas',
            'people.get|Obtener personas',
            'people.edit|Editar personas',
            'people.delete|Eliminar personas',
        ));

        // Catálogos
        $this->addPermissions('CATÁLOGOS', array(
            'catalogs|Ver catálogos',
            'catalogs.create|Registrar catálogos',
            'catalogs.get|Obtener catálogos',
            'catalogs.edit|Editar catálogos',
            'catalogs.delete|Eliminar catálogos',
            // Items
            'catalogs.items|Ver items',
            'catalogs.items.create|Registrar items',
            'catalogs.items.edit|Editar items',
            'catalogs.items.delete|Eliminar items',
            'catalogs.items.import|Importar items',
        ));

        // Usuarios
        $this->addPermissions('USUARIOS', array(
            'users|Ver usuarios',
            'users.create|Registrar usuarios',
            'users.get|Obtener usuarios',
            'users.edit|Editar usuarios',
            'users.toggle-status|Cambiar estatus',
            'users.revoke-session|Revocar la sesión',
        ));

        // Roles
        $this->addPermissions('ROLES', array(
            'roles|Ver roles',
            'roles.create|Registrar roles',
            'roles.get|Obtener roles',
            'roles.edit|Editar roles',
            'roles.delete|Eliminar roles',
        ));

        // Permisos
        $this->addPermissions('PERMISOS', array(
            'permissions|Ver permisos',
            'permissions.create|Registrar permisos',
            'permissions.edit|Editar permisos',
            'permissions.delete|Eliminar permisos',
        ));
    }

    public function addPermissions(String $module, Array $permissionsData)
    {
        try {
            DB::beginTransaction();

            $module = CatItem::firstWhere('name', $module);

            foreach ($permissionsData as $permissionData) {
                $permission = explode('|', $permission);
                $name = $permission[0];
                $tag = $permission[1];

                Permission::create([
                    'name' => $name,
                    'tag' => $tag,
                    'module_id' => $module->id,
                ]);
            }

            DB::commit();
            
        } catch (Throwable $e) {
            DB::rollback();

            return $e->getMessage();
        }
    }
}
