<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\AuthController;
use App\Http\Controllers\ExpenseObjectController;
use App\Http\Controllers\AccountController;
use App\Http\Controllers\TransactionController;
use App\Http\Controllers\EntityController;
use App\Http\Controllers\NodeController;
use App\Http\Controllers\VaultBoxController;
use App\Http\Controllers\PersonController;
use App\Http\Controllers\CatalogController;
use App\Http\Controllers\UserController;
use App\Http\Controllers\RoleController;
use App\Http\Controllers\PermissionController;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider and all of them will
| be assigned to the "api" middleware group. Make something great!
|
*/

Route::prefix('/auth')->group(function () {
    Route::post('/login', [AuthController::class, 'login']);
    Route::post('/logout', [AuthController::class, 'logout'])->middleware('auth:sanctum');
});

Route::middleware('auth:sanctum')->group(function () {
    Route::prefix('/expense-objects')->group(function () {
        Route::get('/', [ExpenseObjectController::class, 'all'])->middleware('permission:expense-objects');
        Route::post('/create', [ExpenseObjectController::class, 'create'])->middleware('permission:expense-objects.create');
        Route::get('/get', [ExpenseObjectController::class, 'get'])->middleware('permission:expense-objects.get');
        Route::put('/edit', [ExpenseObjectController::class, 'edit'])->middleware('permission:expense-objects.edit');
        Route::delete('/delete', [ExpenseObjectController::class, 'delete'])->middleware('permission:expense-objects.delete');
        Route::post('/import', [ExpenseObjectController::class, 'import'])->middleware('permission:expense-objects.import');
    });

    Route::prefix('/accounts')->group(function () {
        Route::get('/', [AccountController::class, 'all']);
        Route::post('/create', [AccountController::class, 'create']);
        Route::put('/edit', [AccountController::class, 'edit']);
        Route::delete('/delete', [AccountController::class, 'delete']);
    });

    Route::prefix('/transactions')->group(function () {
        Route::get('/', [TransactionController::class, 'all'])->middleware('permission:transactions');
        Route::get('/by-range', [TransactionController::class, 'getByVaultBoxAndDate']);
        Route::put('/upload-proof-transaction', [TransactionController::class, 'uploadProofTransaction'])
            /* ->middleware('permission:transactions.upload-proof-payment') */;
        Route::get('/get-proof-transaction', [TransactionController::class, 'getProofTransaction'])
            /* ->middleware('permission:transactions.get-proof-payment') */;
        // Expenses
        Route::post('/expenses/register', [TransactionController::class, 'saveExpense']);
        Route::put('/expenses/edit', [TransactionController::class, 'editExpense']);
        Route::delete('/expenses/delete', [TransactionController::class, 'deleteExpense']);
        Route::post('/expenses/import', [TransactionController::class, 'importExpenses']);
        // Dispersions
        Route::post('/dispersions/register', [TransactionController::class, 'saveDispersion']);
        Route::put('/dispersions/edit', [TransactionController::class, 'editDispersion']);
        Route::delete('/dispersions/delete', [TransactionController::class, 'deleteDispersion']);
        // Items
        Route::get('/{id}/items', [TransactionController::class, 'getItems']);
    });

    Route::prefix('/entities')->group(function () {
        Route::get('/', [EntityController::class, 'all'])->middleware('permission:entities');
        Route::post('/create', [EntityController::class, 'create'])->middleware('permission:entities.create');
        Route::get('/get', [EntityController::class, 'get'])->middleware('permission:entities.get');
        Route::put('/edit', [EntityController::class, 'edit'])->middleware('permission:entities.edit');
        Route::delete('/delete', [EntityController::class, 'delete'])->middleware('permission:entities.delete');
    });

    Route::prefix('/nodes')->group(function () {
        Route::get('/', [NodeController::class, 'all']);
        Route::post('/create', [NodeController::class, 'create']);
        Route::put('/edit', [NodeController::class, 'edit']);
        Route::delete('/delete', [NodeController::class, 'delete']);
    });

    Route::prefix('/vaults-boxes')->group(function () {
        Route::get('/', [VaultBoxController::class, 'all']);
        Route::post('/create', [VaultBoxController::class, 'create']);
        Route::put('/edit', [VaultBoxController::class, 'edit']);
        // Route::delete('/delete', [VaultBoxController::class, 'delete']);
        Route::put('/add-income', [VaultBoxController::class, 'addIncome']);
        Route::put('/toggle-status', [VaultBoxController::class, 'toggleStatus']);
        Route::put('/refund-balance', [VaultBoxController::class, 'refundBalance']);
        // Historial
        Route::get('/{id}/history', [VaultBoxController::class, 'getHistoryByVaultBox']);
        Route::put('/{id}/history/close', [VaultBoxController::class, 'closeHistory']);
        // Responsables
        Route::get('/managers', [VaultBoxController::class, 'getManagers']);
        Route::post('/managers/add', [VaultBoxController::class, 'addManager']);
        Route::delete('/managers/remove', [VaultBoxController::class, 'removeManager']);
        Route::get('/belongs-to/{id}', [VaultBoxController::class, 'getByManager']);
    });

    Route::prefix('/people')->group(function () {
        Route::get('/', [PersonController::class, 'all'])->middleware('permission:people');
        Route::post('/create', [PersonController::class, 'create'])->middleware('permission:people.create');
        Route::put('/edit', [PersonController::class, 'edit'])->middleware('permission:people.edit');
        Route::delete('/delete', [PersonController::class, 'delete'])->middleware('permission:people.delete');
    });

    Route::prefix('/catalogs')->group(function () {
        Route::get('/', [CatalogController::class, 'all'])->middleware('permission:catalogs');
        Route::post('/create', [CatalogController::class, 'create'])->middleware('permission:catalogs.create');
        Route::get('/get', [CatalogController::class, 'get'])->middleware('permission:catalogs.get');
        Route::put('/edit', [CatalogController::class, 'edit'])->middleware('permission:catalogs.edit');
        Route::delete('/delete', [CatalogController::class, 'delete'])->middleware('permission:catalogs.delete');
        // Items
        Route::get('/{id}/items', [CatalogController::class, 'items'])->middleware('permission:catalogs.items');
        Route::post('/items/create', [CatalogController::class, 'createItem'])->middleware('permission:catalogs.items.create');
        Route::put('/items/edit', [CatalogController::class, 'editItem'])->middleware('permission:catalogs.items.edit');
        Route::delete('/items/delete', [CatalogController::class, 'deleteItem'])->middleware('permission:catalogs.items.delete');
        Route::post('/items/import', [CatalogController::class, 'importItems'])->middleware('permission:catalogs.items.import');
    });

    Route::prefix('/users')->group(function () {
        Route::get('/', [UserController::class, 'all'])->middleware('permission:users');
        Route::post('/create', [UserController::class, 'create'])/* ->middleware('permission:users.create') */;
        Route::get('/get', [UserController::class, 'get'])->middleware('permission:users.get');
        Route::put('/edit', [UserController::class, 'edit'])->middleware('permission:users.edit');
        Route::put('/toggle-status', [UserController::class, 'toggleStatus'])->middleware('permission:users.toggle-status');
        Route::put('/change-password', [UserController::class, 'changePassword']);
        Route::put('/upload-profile-photo', [UserController::class, 'uploadProfilePhoto']);
        Route::get('/get-profile-photo', [UserController::class, 'getProfilePhoto']);
        Route::post('/revoke-session', [UserController::class, 'revokeSession'])->middleware('permission:users.revoke-session');
    });

    Route::prefix('/roles')->group(function () {
        Route::get('/', [RoleController::class, 'all'])->middleware('permission:roles');
        Route::post('/create', [RoleController::class, 'create'])->middleware('permission:roles.create');
        Route::get('/get', [RoleController::class, 'get'])->middleware('permission:roles.get');
        Route::put('/edit', [RoleController::class, 'edit'])->middleware('permission:roles.edit');
        Route::delete('/delete', [RoleController::class, 'delete'])->middleware('permission:roles.delete');
    });

    Route::prefix('/permissions')->group(function () {
        Route::get('/', [PermissionController::class, 'all'])->middleware('permission:permissions');
        Route::post('/create', [PermissionController::class, 'create'])->middleware('permission:permissions.create');
        Route::put('/edit', [PermissionController::class, 'edit'])->middleware('permission:permissions.edit');
        Route::delete('/delete', [PermissionController::class, 'delete'])->middleware('permission:permissions.delete');
        Route::get('/by-modules', [PermissionController::class, 'byModules'])->middleware('permission:permissions');
    });

    Route::get('/auth-verification', function () { // Para verificar si existen tokens relacionados al usuario
        return response()->json(['message' => 'you are logged in.'], 200);
    });
});